#include <iostream>
#include <fstream>
#include <cmath>
#include <string>

const int INPUT_SIZE = 8;
const int HIDDEN_SIZE = 5;
const int OUTPUT_SIZE = 1;
const int EPOCHS = 1000;
const double LEARNING_RATE = 0.01;

const double INITIAL_WEIGHT = 0.5;
const double INITIAL_BIAS = 0.5;

double sigmoid(double x)
{
    return 1.0 / (1.0 + std::exp(-x));
}

double sigmoid_derivative(double x)
{
    double sigmoid_x = sigmoid(x);
    return sigmoid_x * (1 - sigmoid_x);
}

class Perceptron
{
public:
    double weights_input_hidden[INPUT_SIZE][HIDDEN_SIZE];
    double weights_hidden_output[HIDDEN_SIZE][OUTPUT_SIZE];
    double bias_hidden[HIDDEN_SIZE];
    double bias_output[OUTPUT_SIZE];

    Perceptron()
    {
        for (int i = 0; i < INPUT_SIZE; ++i)
        {
            for (int j = 0; j < HIDDEN_SIZE; ++j)
            {
                weights_input_hidden[i][j] = INITIAL_WEIGHT;
            }
        }
        for (int j = 0; j < HIDDEN_SIZE; ++j)
        {
            bias_hidden[j] = INITIAL_BIAS;
        }
        for (int j = 0; j < HIDDEN_SIZE; ++j)
        {
            for (int k = 0; k < OUTPUT_SIZE; ++k)
            {
                weights_hidden_output[j][k] = INITIAL_WEIGHT;
            }
        }
        for (int k = 0; k < OUTPUT_SIZE; ++k)
        {
            bias_output[k] = INITIAL_BIAS;
        }
    }

    void train(double data[][INPUT_SIZE], int labels[], int num_samples)
    {
        for (int epoch = 0; epoch < EPOCHS; ++epoch)
        {
            for (int i = 0; i < num_samples; ++i)
            {
                double hidden_layer[HIDDEN_SIZE];
                double output_layer[OUTPUT_SIZE];

                for (int j = 0; j < HIDDEN_SIZE; ++j)
                {
                    hidden_layer[j] = 0;
                    for (int k = 0; k < INPUT_SIZE; ++k)
                    {
                        hidden_layer[j] += data[i][k] * weights_input_hidden[k][j];
                    }
                    hidden_layer[j] += bias_hidden[j];
                    hidden_layer[j] = sigmoid(hidden_layer[j]);
                }
                for (int k = 0; k < OUTPUT_SIZE; ++k)
                {
                    output_layer[k] = 0;
                    for (int j = 0; j < HIDDEN_SIZE; ++j)
                    {
                        output_layer[k] += hidden_layer[j] * weights_hidden_output[j][k];
                    }
                    output_layer[k] += bias_output[k];
                    output_layer[k] = sigmoid(output_layer[k]);
                }
            }
        }
    }

    void save_weights(const std::string &folder_path, const std::string &filename)
    {
        std::string full_path = folder_path + "/" + filename;
        std::ofstream file(full_path, std::ios::binary);
        if (file.is_open())
        {
            file.write((char *)weights_input_hidden, sizeof(weights_input_hidden));
            file.write((char *)weights_hidden_output, sizeof(weights_hidden_output));
            file.write((char *)bias_hidden, sizeof(bias_hidden));
            file.write((char *)bias_output, sizeof(bias_output));
            file.close();
        }
        else
        {
            std::cerr << "Error opening file: " << full_path << std::endl;
        }
    }
};

int main()
{
    double data[1][INPUT_SIZE] = {{6, 148, 72, 35, 0, 33.6, 0.627, 50}};
    int labels[1] = {1};

    Perceptron p;
    p.train(data, labels, 1);
    p.save_weights("/home/user/data/", "weights_node05.bin"); // Update this path
    
    return 0;
}
