#include <iostream>
#include <mpi.h>
#include <cstdlib>
#include <ctime>

const int SIZE = 1000;

void multiplyMatrices(int *A, int *B, int *C, int rows, int size)
{
    for (int i = 0; i < rows; ++i)
    {
        for (int j = 0; j < size; ++j)
        {
            C[i * size + j] = 0;
            for (int k = 0; k < size; ++k)
            {
                C[i * size + j] += A[i * size + k] * B[k * size + j];
            }
        }
    }
}

int main(int argc, char **argv)
{
    MPI_Init(&argc, &argv);

    int world_rank, world_size;
    MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &world_size);

    int rows_per_process = SIZE / world_size;

    int *A = new int[rows_per_process * SIZE];
    int *B = new int[SIZE * SIZE];
    int *C = new int[rows_per_process * SIZE];

    if (world_rank == 0)
    {
        int *full_A = new int[SIZE * SIZE];
        int *full_B = new int[SIZE * SIZE];

        for (int i = 0; i < SIZE; ++i)
        {
            for (int j = 0; j < SIZE; ++j)
            {
                full_A[i * SIZE + j] = i + j;
                full_B[i * SIZE + j] = i - j;
            }
        }

        MPI_Scatter(full_A, rows_per_process * SIZE, MPI_INT, A, rows_per_process * SIZE, MPI_INT, 0, MPI_COMM_WORLD);

        MPI_Bcast(full_B, SIZE * SIZE, MPI_INT, 0, MPI_COMM_WORLD);

        for (int i = 0; i < SIZE * SIZE; ++i)
        {
            B[i] = full_B[i];
        }

        delete[] full_A;
        delete[] full_B;
    }
    else
    {
        MPI_Scatter(nullptr, 0, MPI_DATATYPE_NULL, A, rows_per_process * SIZE, MPI_INT, 0, MPI_COMM_WORLD);

        MPI_Bcast(B, SIZE * SIZE, MPI_INT, 0, MPI_COMM_WORLD);
    }

    clock_t start = clock();

    multiplyMatrices(A, B, C, rows_per_process, SIZE);

    clock_t end = clock();

    int *full_C = nullptr;
    if (world_rank == 0)
    {
        full_C = new int[SIZE * SIZE];
    }
    MPI_Gather(C, rows_per_process * SIZE, MPI_INT, full_C, rows_per_process * SIZE, MPI_INT, 0, MPI_COMM_WORLD);

    if (world_rank == 0)
    {
        double time_taken = double(end - start) / CLOCKS_PER_SEC;
        std::cout << "Time taken for matrix multiplication: " << time_taken << " seconds" << std::endl;

        // Uncomment the following block to print the result matrix
        /*
        std::cout << "Result matrix C:" << std::endl;
        for (int i = 0; i < SIZE; ++i)
        {
            for (int j = 0; j < SIZE; ++j)
            {
                std::cout << full_C[i * SIZE + j] << " ";
            }
            std::cout << std::endl;
        }
        */
        delete[] full_C;
    }

    delete[] A;
    delete[] B;
    delete[] C;

    MPI_Finalize();
    return 0;
}
