#include <iostream>
#include <mpi.h>
#include <cstdlib>
#include <ctime>

const int NUM_POINTS = 10000000;

int main(int argc, char **argv)
{
    MPI_Init(&argc, &argv);

    int world_rank, world_size;
    MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
    MPI_Comm_size(MPI_COMM_WORLD, &world_size);

    std::srand(std::time(nullptr) + world_rank);

    int points_per_process = NUM_POINTS / world_size;
    int inside_circle = 0;

    clock_t start = clock();
    for (int i = 0; i < points_per_process; ++i)
    {
        double x = static_cast<double>(std::rand()) / RAND_MAX;
        double y = static_cast<double>(std::rand()) / RAND_MAX;

        if (x * x + y * y <= 1.0)
        {
            ++inside_circle;
        }
    }

    clock_t end = clock();

    int total_inside_circle = 0;
    MPI_Reduce(&inside_circle, &total_inside_circle, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD);

    if (world_rank == 0)
    {
        double pi_estimate = 4.0 * total_inside_circle / NUM_POINTS;
        double duration = double(end - start) / CLOCKS_PER_SEC;
        std::cout << "Estimated value of Pi: " << pi_estimate << std::endl;
        std::cout << "Time taken: " << duration << " seconds" << std::endl;
    }

    MPI_Finalize();
    return 0;
}
